//+------------------------------------------------------------------+
//|                                                   Pip Target.mq4 |
//|                                                 Copyright  2010 |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 YellowGreen
#property indicator_width1 3

//--- input parameters
extern string TimeFrame       = "Current time frame";
extern int    ATRPeriod		  = 12;
extern int 	  HistoryLength   = 60;
//--- buffers
double PipTarget[];
double ATRBuffer[];

// Globals
string indicatorFileName;
int    timeFrame;
bool   first;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorBuffers(2);
   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(0,PipTarget);
   SetIndexBuffer(1,ATRBuffer);
//----
   HistoryLength = MathMax(HistoryLength,1);
   ATRPeriod     = MathMax(ATRPeriod,1);
   first=true;
   
   //
   //
   indicatorFileName = WindowExpertName();
   timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   IndicatorShortName(timeFrameToString(timeFrame)+" ("+ATRPeriod+")"+" ("+HistoryLength+")");
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int    counted_bars=IndicatorCounted();
//----
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;

   int step           = timeFrame/Period();
   int initialDrawBar = Bars-1-(HistoryLength+ATRPeriod)*step;
   
   if (initialDrawBar < 0) { return(0); }
           limit=MathMin(Bars-counted_bars, initialDrawBar);
           double mtfLimit = MathMin(limit+1, initialDrawBar);
           //if (returnBars) { ATRBuffer[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   mtfLimit = MathMax(limit,MathMin(Bars-1-initialDrawBar, mtfLimit*step));
   
   if (first) {
   		Print ("Bars = ", DoubleToStr(Bars, 0)); 
   		Print ("limit = ", DoubleToStr(limit, 0));
   		Print ("mtfLimit = ", DoubleToStr(mtfLimit, 0)); 
   		Print ("Hist*Step = ", DoubleToStr(HistoryLength*step, 0));
   		//first=false;
   		}

   for (i=mtfLimit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
      ATRBuffer[i] = iATR(NULL,timeFrame,ATRPeriod,y);
   }

   //
   //
   int outerLimit = MathMin(Bars-1-(ATRPeriod/*+25*/)*step, limit+HistoryLength*step);
   //int outerLimit = MathMin(mtfLimit, limit+HistoryLength*step);
   double ATRMin = ATRBuffer[outerLimit]; 
   double ATRMax = ATRBuffer[outerLimit];
   if (first) {
   		Print ("outerLimit = ", DoubleToStr(outerLimit, 0)); 
   		Print ("ATRMin = ", DoubleToStr(ATRMin, Digits)); 
   		Print ("ATRMax = ", DoubleToStr(ATRMax, Digits));
   		//first=false;
   		}
   limit = MathMax(0,(outerLimit-HistoryLength*step));
   for (i=limit; i>=0; i--) {
   		for (int k=i+HistoryLength*step; k>=i; k=k-step) {
   				if (ATRBuffer[k]==0)
   					break;
   				ATRMin = MathMin(ATRMin, ATRBuffer[k]);
   				ATRMax = MathMax(ATRMax, ATRBuffer[k]);
   			}
   		PipTarget[i]=ATRMax+ATRMin-ATRBuffer[i];
   		}

   if (first) {
   		Print ("AfterMin = ", DoubleToStr(ATRMin, Digits)); 
   		Print ("AfterMax = ", DoubleToStr(ATRMax, Digits));
   		first=false;
   		}	
//----
   return(0);
  }
//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}
//+------------------------------------------------------------------+